(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
This is the Renapp.m file, to be used in the FeynRules environment.

This is where we define everything related to the renormalization in the FeynRules environment.

Created by: Duarte Fontes
Email: duartefontes@tecnico.ulisboa.pt
Last update: 06.09.2019
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

Clear[p]; Clear[r];
(* Set the derivatives of the renormalization constants to zero *)
Do[consrules = Append[consrules, del[renconsadd[[i]], \[Mu]_] -> 0], {i, 1, Length[renconsadd]}];

(* Preparing the Lagrangean for the counterterms *)
L2S = 0;
If[ToString[InputForm[LGauge]] != "LGauge", L2S += ExpandIndices[LGauge]];
(* Note: I cannot apply ExpandIndices to LHiggs since this function uses (at least some) of the restrictions
of the restrictions file; and these restrictions cannot be used here, for they assume tad = 0 *)
If[ToString[InputForm[LHiggs]] != "LHiggs", L2S += LHiggs /. SU2exp];
If[ToString[InputForm[LGhost]] != "LGhost", L2S += ExpandIndices[LGhost]];
If[GFreno,If[ToString[InputForm[LGF]] != "LGF", L2S += ExpandIndices[LGF]]];
If[ToString[InputForm[LFermions]] != "LFermions", L2S += LFermions /. SU2exp];
If[ToString[InputForm[LYukawa]] != "LYukawa", L2S += LYukawa];
L2S = Expand[L2S];

(* Expand the renormalization rules: *)
renorrules = renorrules // Expand;
(* weakrep contains some replacement rules due to counterterms that appear in denominators *)
weakrep = {
	1/(y_ + y_*x_) /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + y*x), {x, 0, 1}]],
    1/(y_ + y_*x_)^2 /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + y*x)^2, {x, 0, 1}]],
    1/(y_ + y_*x_)^3 /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + y*x)^3, {x, 0, 1}]],
    1/(y_ + x_) /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + x), {x, 0, 1}]],
    1/(y_ + x_)^2 /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + x)^2, {x, 0, 1}]],
    1/(y_ + x_)^3 /; StringTake[ToString[x // InputForm], 1] == "\[Delta]" -> Normal[Series[1/(y + x)^3, {x, 0, 1}]] };
(* Insert an \[Eta]\[Delta] for each \[Delta] constant: *)
insetdel = {};
Do[insetdel = Append[insetdel, renconsadd[[i]] -> \[Eta]\[Delta] renconsadd[[i]]], {i, 1, Length[renconsadd]}];
(* Define \[Eta]\[Delta] as a (real) number, not a matrix: *)
nomat = {HC[\[Eta]\[Delta]] -> \[Eta]\[Delta]};

(* Take the non-matricial Deltas out of the fermionic products: *)
repsdeltas={};
Do[
repsdeltas = Append[repsdeltas, (renconscar[[i]] x_) . y_ -> renconscar[[i]] (x.y)];
repsdeltas = Append[repsdeltas, x_ . (renconscar[[i]] y_) -> renconscar[[i]] (x.y)],{i,1,Length[renconscar]}];

(* Put Dirac slashed in a pretty form *)
gamrep = {Ga[Index[Lorentz, mu_]].ProjM*fv[p_, mu_] -> DiracGamma[Momentum[p]].ProjM,\
					Ga[Index[Lorentz, mu_]].ProjP*fv[p_, mu_] -> DiracGamma[Momentum[p]].ProjP,\
				     Ga[Index[Lorentz, mu_]]*fv[p_, mu_] -> DiracGamma[Momentum[p]]};

(* Rules to get rid of the annoying internal FeynRules indices: *)
indrep = {"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					 "[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _~~ _~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Generation, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",
					"[Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Generation, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "], Index[Colour, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]]" -> "",  
   					", Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ "]" -> "",
   					", Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _~~ _ ~~"], Index[Spin, " ~~ _ ~~ _ ~~ _ ~~ _ ~~ _ ~~ _~~ _ ~~ "]" -> ""};

(* Rules to create the Feynman rules from tadpoles: *)
tadrep={};
Do[If[palist[[i, 1]] == "S",tadrep = Append[tadrep, ToExpression[palist[[i, 2]]] -> I]], {i,1,Length[palist]}];

GetCTpre=Function[exp,Module[{},
(* Chain of operations to obtain the Lagrangean for the counterterms: *)
LwCTpre = ((exp /. rensimp /. rensimp) // Simplify) /. rensimp;
LwCTpre2 = (LwCTpre /. renorrules /. weakrep /. consrules) // Expand;
LwCTpre3 = Expand[LwCTpre2 /. weakrep];
LwCT = LwCTpre3 /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa /. dotexpa;
LwCT2 = LwCT /. insetdel /. nomat;
LwCT3 = Expand[LwCT2 /. repsetas /. repsetas /. repsetas /. repsetas /. repsetas /. repsetas /. repsetas /. repsetas];
LwCT4 = Expand[LwCT3 /. repsdeltas /. repsdeltas /. repsdeltas /. repsdeltas /. repsdeltas];
LoCTpre = Coefficient[LwCT4, \[Eta]\[Delta], 1];
LoCT = WeylToDirac[LoCTpre];
LoCT2 = ExpandIndices[LoCT];
LoCT3 = StringReplace[ToString[LoCT2 // InputForm], indrep];
LoCT4 = ToExpression[LoCT3];
LoCT5 = Expand[LoCT4 /. notensdots /. todots];
LoCT6 = (LoCT5 /. insetas) /. consrules;
LoCT7 = Expand[LoCT6 /. repsetas /. repsetas /. repsetas /. repsetas /. repsetas /. repsdeltas /. repsdeltas /. repsdeltas /. repsdeltas];
LoCT8 = LoCT7 /. stilletas;
LoCT9 = LoCT8 /. consrules;
(* - - specific operations for propagators and tadpoles: - - *)
LoCTproppre1 = ExpandIndices[LoCT9, MaxParticles -> 2];
LoCTproppre2 = StringReplace[ToString[LoCTproppre1 // InputForm], indrep];
LoCTproppre3 = ToExpression[LoCTproppre2];
LoCTprop = Expand[LoCTproppre3 /. proprep];
LoCTpropntad = Expand[LoCTprop /. tadrep];
(* - - specific operations for vertices: - - *)
ancCTve1 = ExpandIndices[LoCT9, MinParticles -> 3];
ancCTve2 = FeynmanRules[ancCTve1];
LoCTvert = 0;
Do[LoCTvert += (Simplify[ancCTve2[[i,2]]] /. MyRestr /. MyRestr),{i,1,Length[ancCTve2]}];
LoCTvert=Expand[LoCTvert];
(* - - final Lagrangean for the counterterms: - - *)
LoCTans = LoCTpropntad + LoCTvert
]];

GetCT=Function[exp,Module[{}, GetCTpre[exp] /. unitetas]];

LoCTfin = GetCTpre[L2S];

(* Identify all the combinations of \[Eta]s that appear in the final Lagrangean for the counterterms: *)
LoCTlist=List @@ LoCTfin;
etaux1=0;
etpre={};
Do[Do[
If[ToString[InputForm[Exponent[LoCTlist[[j]],fixetas[[i]]]]] != "0", 
If[ToString[InputForm[etaux1]]=="0",
etaux1=Power[fixetas[[i]],Exponent[LoCTlist[[j]],fixetas[[i]]]],
etaux1=etaux1*Power[fixetas[[i]],Exponent[LoCTlist[[j]],fixetas[[i]]]]]],{i,1,Length[fixetas]}];
If[ToString[InputForm[etaux1]]!="0",etpre=Append[etpre,etaux1]];
etaux1=0,{j,1,Length[LoCTlist]}];
combetas={};
Do[If[! MemberQ[combetas,etpre[[i]]], combetas=Append[combetas,etpre[[i]]]],{i,1,Length[etpre]}];

(* Write the final list of counterterms: *)
CTppplist = {};
Do[CTppplist = Append[CTppplist,(((Coefficient[LoCTfin, combetas[[i]]] /.cleanetas)* combetas[[i]]) /.gamrep // Simplify) /.pprops /. assodots // InputForm], {i, 1, Length[combetas]}];
CTinilist=CTppplist /.MyRestr;
Clear[i];

(* Some details to include in the external file where CTinilist is to be written: *)
VList := ancCTve2
VoutL = {};
Do[
		auxVout = {};
		Do[auxVout = Append[auxVout,VList[[i,1,j,1]]],{j,1,Length[VList[[i,1]]]}];
		VoutL = Append[VoutL,auxVout];
		VoutL = VoutL /. FRtoQ,{i,1,Length[VList]}];
VoutL2 = {};
Do[VoutL2 = Append[VoutL2, {}], {i, 1, Length[VoutL]}];
Do[Do[VoutL2[[i]] = Append[VoutL2[[i]], ToString[InputForm[VoutL[[i, j]]]]], {j, 1, Length[VoutL[[i]]]}], {i, 1, Length[VoutL]}];
AoutL = {};
Do[Do[If[ToString[FRtoQ[[i, 1]] // InputForm] <> "bar" == ToString[FRtoQ[[j, 1]] // InputForm],
						AoutL = Append[AoutL,FRtoQ[[i,2]] -> FRtoQ[[j,2]]];
						AoutL = Append[AoutL,FRtoQ[[j,2]] -> FRtoQ[[i,2]]]],{j,1,Length[FRtoQ]}],{i,1,Length[FRtoQ]}];

CTpreord1 = combetas /. comeback;
CTpreord2 = {};
Do[ordaux = {};
  		If[Length[CTpreord1[[i]]] == 0,
   				ordaux = Append[ordaux, ToString[CTpreord1[[i]]]], 
   				If[Length[CTpreord1[[i]]] == 2 && IntegerQ[CTpreord1[[i, 2]]], 
    				Do[ordaux = Append[ordaux, ToString[CTpreord1[[i, 1]]]], {m, 1, CTpreord1[[i, 2]]}], 
    				Do[If[Length[CTpreord1[[i, j]]] > 0,
      							Do[ordaux = Append[ordaux, ToString[CTpreord1[[i, j, 1]]]], {k, 1, CTpreord1[[i, j, 2]]}],
      							ordaux = Append[ordaux, ToString[CTpreord1[[i, j]]]]], {j, 1, Length[CTpreord1[[i]]]}]]];
  		CTpreord2 = Append[CTpreord2, ordaux], {i, 1, Length[CTpreord1]}];
CTord = CTpreord2;
Do[Do[If[ContainsAll[CTord[[i]], VoutL2[[j]]] && ContainsAll[VoutL2[[j]] , CTord[[i]]] && Length[CTord[[i]]] == Length[VoutL2[[j]]],
					CTord[[i]] = VoutL2[[j]]], {j, 1, Length[VoutL2]}], {i, 1, Length[CTord]}];

(* Finally, the external file to the FeynCalc envirnonment: *)
str = OpenWrite["CTpre.m"];
Do[WriteString[str, "+++++ "<>ToString[CTinilist[[i]]]<>"\n"],{i,1,Length[CTinilist]}];
WriteString[str, "\n"];
WriteString[str, "CTord := "];
WriteString[str, ToString[CTord]];
WriteString[str, "\n"];
WriteString[str, "cleanetas := "];
WriteString[str, ToString[cleanetas]];
WriteString[str, "\n"];
WriteString[str, "antietas := "];
WriteString[str, ToString[antietas]];
WriteString[str, "\n"];
WriteString[str, "convetas := "];
WriteString[str, ToString[convetas]];
WriteString[str, "\n"];
WriteString[str, "VertLi := "];
WriteString[str, ToString[VoutL]];
WriteString[str, "\n"];
WriteString[str, "AntiRep := "];
WriteString[str, ToString[AoutL]];
Close[str];

(* Now, we focus on the printing of the Feynman rules for the counterterms. *)
(* We start by creating some (more) lists, which will be very useful for the printing *)
palistbri1 = {};
Do[palistbri1 = Append[palistbri1, {palist2[[i, 2]]}];
  If[ToString[StandardForm[ToExpression[ToString[palist2[[i, 2]]] <> "bar"]]] != ToString[InputForm[ToExpression[ToString[palist2[[i, 2]]] <> "bar"]]], 
   palistbri1 = Append[palistbri1, {ToExpression[ToString[palist2[[i, 2]]] <> "bar"]}]], {i, 1, Length[palist2]}];
palistbri2 = {};
Do[palistbri2 = Append[palistbri2, {ToString[InputForm[palistbri1[[i, 1]]]]}], {i, 1, Length[palistbri1]}];
Do[If[StringLength[palistbri2[[i, 1]]] > 2 && StringTake[palistbri2[[i, 1]], -3] == "bar", 
   	   Do[If[StringDelete[palistbri2[[i, 1]], "bar"] == ToString[InputForm[palist2[[k, 2]]]], 
     				palistbri2[[i]] = Append[palistbri2[[i]], ToString[InputForm[palist2[[k, 1]]]]]; 
     				Break[]], {k, 1, Length[palist2]}], 
   		Do[If[palistbri2[[i, 1]] == ToString[InputForm[palist2[[k, 2]]]], 
                   palistbri2[[i]] = Append[palistbri2[[i]], ToString[InputForm[palist2[[k, 1]]]]]; 
     			   Break[]], {k, 1, Length[palist2]}]], {i, 1, Length[palistbri2]}];
palistbri3 = palistbri2 // ToExpression;
palistbri4 = palistbri3 /. FRtoQ;
palistbri5 = {};
Do[palistbri5 = Append[palistbri5, {}], {i, 1, Length[palistbri4]}];
Do[palistbri5[[i]] = Append[palistbri5[[i]], ToString[palistbri4[[i, 1]]]];
       palistbri5[[i]] = Append[palistbri5[[i]], ToString[palistbri4[[i, 2]]]], {i, 1, Length[palistbri4]}];

CTbri1 = {};
Do[CTbri1 = Append[CTbri1, {}], {i, 1, Length[CTord]}];
Do[Do[Do[If[CTord[[i, j]] == palistbri5[[k, 1]], CTbri1[[i]] = Append[CTbri1[[i]], palistbri5[[k, 2]]]], {k, 1, Length[palistbri5]}], {j, 1, Length[CTord[[i]]]}], {i, 1, Length[CTord]}];
CTbri2 = {};
Do[CTbri2 = Append[CTbri2, {}], {i, 1, Length[CTbri1]}];
Do[Do[If[! MemberQ[CTbri2[[i]], CTbri1[[i, j]]], CTbri2[[i]] = Append[CTbri2[[i]], CTbri1[[i, j]]]], {j, 1, Length[CTbri1[[i]]]}], {i, 1, Length[CTbri1]}];

CTmidlist1 = {};
Do[CTmidlist1 = Append[CTmidlist1, ToString[CTinilist[[i]]]], {i, 1, Length[CTinilist]}];
CTmidlist2 = CTmidlist1 // ToExpression;
CTmidlist3 = (CTmidlist2 /. unitetas) /. MyRestr /. MyRestr;
CTmidlist4 = {};
Do[CTmidlist4 = Append[CTmidlist4, {ToString[InputForm[CTmidlist3[[i]]]]}], {i, 1, Length[CTmidlist3]}];

(* Now comes the printing part itself. We divide it by sections *)

(* PrePrintCTRulesv1Tads *)
PrePrintCTRulesv1Tads = {};
Do[If[Length[CTord[[i]]] == 1,
            PrePrintCTRulesv1Tads = Append[PrePrintCTRulesv1Tads, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 1,
   			coin = coin + 1;
   			PrePrintCTRulesv1Tads[[coin]] = Append[PrePrintCTRulesv1Tads[[coin]], CTord[[i]]];
   			PrePrintCTRulesv1Tads[[coin]] = Append[PrePrintCTRulesv1Tads[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv1Tads != {},
  	str1 = OpenWrite["PrePrintCTRulesv1Tads.m"];
    Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv1Tads[[i, 1, 1]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv1Tads[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv1Tads]}];
  	Close[str1]];

(* PrePrintCTRulesv2Props *)
PrePrintCTRulesv2Props = {};
Do[If[Length[CTord[[i]]] == 2,
            PrePrintCTRulesv2Props = Append[PrePrintCTRulesv2Props, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 2,
   			coin = coin + 1;
   			PrePrintCTRulesv2Props[[coin]] = Append[PrePrintCTRulesv2Props[[coin]], CTord[[i]]];
   			PrePrintCTRulesv2Props[[coin]] = Append[PrePrintCTRulesv2Props[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv2Props != {},
  	str1 = OpenWrite["PrePrintCTRulesv2Props.m"];
    Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv2Props[[i, 1, 1]], ",", PrePrintCTRulesv2Props[[i, 1, 2]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv2Props[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv2Props]}];
  	Close[str1]];

(* PrePrintCTRulesv3Gauge *)
PrePrintCTRulesv3Gauge = {};
Do[If[Length[CTord[[i]]] == 3 && CTbri2[[i]] == {"V"},
            PrePrintCTRulesv3Gauge = Append[PrePrintCTRulesv3Gauge, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 3 && CTbri2[[i]] == {"V"},
   			coin = coin + 1;
   			PrePrintCTRulesv3Gauge[[coin]] = Append[PrePrintCTRulesv3Gauge[[coin]], CTord[[i]]];
   			PrePrintCTRulesv3Gauge[[coin]] = Append[PrePrintCTRulesv3Gauge[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv3Gauge != {},
  	str1 = OpenWrite["PrePrintCTRulesv3Gauge.m"];
    Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv3Gauge[[i, 1, 1]], ",", PrePrintCTRulesv3Gauge[[i, 1, 2]], ",", PrePrintCTRulesv3Gauge[[i, 1, 3]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv3Gauge[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv3Gauge]}];
  	Close[str1]];

(* PrePrintCTRulesv4Gauge *)
PrePrintCTRulesv4Gauge = {};
Do[If[Length[CTord[[i]]] == 4 && CTbri2[[i]] == {"V"},
   			PrePrintCTRulesv4Gauge = Append[PrePrintCTRulesv4Gauge, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 4 && CTbri2[[i]] == {"V"},
   			coin = coin + 1;
   			PrePrintCTRulesv4Gauge[[coin]] = Append[PrePrintCTRulesv4Gauge[[coin]], CTord[[i]]];
   			PrePrintCTRulesv4Gauge[[coin]] = Append[PrePrintCTRulesv4Gauge[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv4Gauge != {},
  	str1 = OpenWrite["PrePrintCTRulesv4Gauge.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv4Gauge[[i, 1, 1]], ",", PrePrintCTRulesv4Gauge[[i, 1, 2]], ",", PrePrintCTRulesv4Gauge[[i, 1, 3]], ",", PrePrintCTRulesv4Gauge[[i, 1, 4]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv4Gauge[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv4Gauge]}];
  Close[str1]];

(* PrePrintCTRulesv3Fermions *)
PrePrintCTRulesv3Fermions = {};
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "S"],
   			PrePrintCTRulesv3Fermions = Append[PrePrintCTRulesv3Fermions, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "S"],
   			coin = coin + 1;
   			PrePrintCTRulesv3Fermions[[coin]] = Append[PrePrintCTRulesv3Fermions[[coin]], CTord[[i]]];
   			PrePrintCTRulesv3Fermions[[coin]] = Append[PrePrintCTRulesv3Fermions[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv3Fermions != {},
  	str1 = OpenWrite["PrePrintCTRulesv3Fermions.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv3Fermions[[i, 1, 1]], ",", PrePrintCTRulesv3Fermions[[i, 1, 2]], ",", PrePrintCTRulesv3Fermions[[i, 1, 3]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv3Fermions[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv3Fermions]}];
  Close[str1]];

(* PrePrintCTRulesv3Yukawa *)
PrePrintCTRulesv3Yukawa = {};
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "F"] && MemberQ[CTbri2[[i]], "S"],
   			PrePrintCTRulesv3Yukawa = Append[PrePrintCTRulesv3Yukawa, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "F"] && MemberQ[CTbri2[[i]], "S"],
   			coin = coin + 1;
   			PrePrintCTRulesv3Yukawa[[coin]] = Append[PrePrintCTRulesv3Yukawa[[coin]], CTord[[i]]];
   			PrePrintCTRulesv3Yukawa[[coin]] = Append[PrePrintCTRulesv3Yukawa[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv3Yukawa != {},
  	str1 = OpenWrite["PrePrintCTRulesv3Yukawa.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv3Yukawa[[i, 1, 1]], ",", PrePrintCTRulesv3Yukawa[[i, 1, 2]], ",", PrePrintCTRulesv3Yukawa[[i, 1, 3]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv3Yukawa[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv3Yukawa]}];
  Close[str1]];

(* PrePrintCTRulesv3Higgs *)
PrePrintCTRulesv3Higgs = {};
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "S"] && ! MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "U"],
   PrePrintCTRulesv3Higgs = Append[PrePrintCTRulesv3Higgs, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "S"] && ! MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "U"],
   			coin = coin + 1;
   			PrePrintCTRulesv3Higgs[[coin]] = Append[PrePrintCTRulesv3Higgs[[coin]], CTord[[i]]];
   			PrePrintCTRulesv3Higgs[[coin]] = Append[PrePrintCTRulesv3Higgs[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv3Higgs != {},
  	str1 = OpenWrite["PrePrintCTRulesv3Higgs.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv3Higgs[[i, 1, 1]], ",", PrePrintCTRulesv3Higgs[[i, 1, 2]], ",", PrePrintCTRulesv3Higgs[[i, 1, 3]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv3Higgs[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv3Higgs]}];
  Close[str1]];

(* PrePrintCTRulesv4Higgs *)
PrePrintCTRulesv4Higgs = {};
Do[If[Length[CTord[[i]]] == 4 && (CTbri2[[i]] == {"S"} || (MemberQ[CTbri2[[i]], "S"] && MemberQ[CTbri2[[i]], "V"])),
   			PrePrintCTRulesv4Higgs = Append[PrePrintCTRulesv4Higgs, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 4 && (CTbri2[[i]] == {"S"} || (MemberQ[CTbri2[[i]], "S"] && MemberQ[CTbri2[[i]], "V"])),
   			coin = coin + 1;
   			PrePrintCTRulesv4Higgs[[coin]] = Append[PrePrintCTRulesv4Higgs[[coin]], CTord[[i]]];
   			PrePrintCTRulesv4Higgs[[coin]] = Append[PrePrintCTRulesv4Higgs[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv4Higgs != {},
  	str1 = OpenWrite["PrePrintCTRulesv4Higgs.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv4Higgs[[i, 1, 1]], ",", PrePrintCTRulesv4Higgs[[i, 1, 2]], ",", PrePrintCTRulesv4Higgs[[i, 1, 3]], ",", PrePrintCTRulesv4Higgs[[i, 1, 4]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv4Higgs[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv4Higgs]}];
  Close[str1]];  

(* PrePrintCTRulesv3Ghosts *)
PrePrintCTRulesv3Ghosts = {};
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "U"],
   			PrePrintCTRulesv3Ghosts = Append[PrePrintCTRulesv3Ghosts, {}]], {i, 1, Length[CTord]}];
coin = 0;
Do[If[Length[CTord[[i]]] == 3 && MemberQ[CTbri2[[i]], "U"],
   			coin = coin + 1;
   			PrePrintCTRulesv3Ghosts[[coin]] = Append[PrePrintCTRulesv3Ghosts[[coin]], CTord[[i]]];
   			PrePrintCTRulesv3Ghosts[[coin]] = Append[PrePrintCTRulesv3Ghosts[[coin]], CTmidlist4[[i]]]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesv3Ghosts != {},
  	str1 = OpenWrite["PrePrintCTRulesv3Ghosts.m"];
  	Do[WriteString[str1, "{{{{{{", PrePrintCTRulesv3Ghosts[[i, 1, 1]], ",", PrePrintCTRulesv3Ghosts[[i, 1, 2]], ",", PrePrintCTRulesv3Ghosts[[i, 1, 3]], "}}}}}},{{{{{{ ", 
    		PrePrintCTRulesv3Ghosts[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesv3Ghosts]}];
  Close[str1]];