(******************* Particle Classes *******************)

M$ClassesDescription = {
(*************** Weyl spinors ***************)
  W[1] == {
		ClassName -> uqR,
		ClassMembers -> {uR, cR, tR},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		QuantumNumbers -> {Q -> 2/3},
		Chirality -> Right},
  W[2] == {
		ClassName -> uqL,
		ClassMembers -> {uL, cL, tL},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		QuantumNumbers -> {Q -> 2/3},
		Chirality -> Left},
  W[3] == {
		ClassName -> dqR,
		ClassMembers -> {dR, sR, bR},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		QuantumNumbers -> {Q -> -1/3},
		Chirality -> Right},
  W[4] == {
		ClassName -> dqL,
		ClassMembers -> {dL, sL, bL},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		QuantumNumbers -> {Q -> -1/3},
		Chirality -> Left},
  W[5] == {
		ClassName -> lR,		
		ClassMembers -> {eR, mR, ttR},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		QuantumNumbers -> {Q -> -1, LeptonNumber -> 1},
		Chirality -> Right},
  W[6] == {
		ClassName -> lL,
		ClassMembers -> {eL, mL, ttL},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		QuantumNumbers -> {Q -> -1, LeptonNumber -> 1},
		Chirality -> Left},
  W[7] == {
		ClassName -> vlL,
		ClassMembers -> {veL,vmL,vtL},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		QuantumNumbers -> {LeptonNumber -> 1},
		Chirality -> Left},
(* even knowing that there are no right-handed neutrinos, I define them, for consistency *)
  W[8] == {
		ClassName -> vlR,
		ClassMembers -> {veR,vmR,vtR},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		QuantumNumbers -> {LeptonNumber -> 1},
		Chirality -> Right},

(*************** Fermions ***************)
  F[1] == {
		ClassName -> vl,
		ClassMembers -> {ve,vm,vt},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		Mass -> {0,0,0},
		QuantumNumbers -> {LeptonNumber -> 1},
		WeylComponents -> {vlL,vlR}},
  F[2] == {
		ClassName -> l,
		ClassMembers -> {e, m, tt},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation]},
		Mass -> {me,mm,mtt},
		QuantumNumbers -> {Q -> -1, LeptonNumber -> 1},
		WeylComponents -> {lL,lR}},
  F[3] == {
		ClassName -> uq,
		ClassMembers -> {u, c, t},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		Mass -> {mu,mc,mt},
		QuantumNumbers -> {Q -> 2/3},
		WeylComponents -> {uqL,uqR}},
  F[4] == {
		ClassName -> dq,		
		ClassMembers -> {d, s, b},
		FlavorIndex -> Generation,
		SelfConjugate -> False,
		Indices -> {Index[Generation], Index[Colour]},
		Mass -> {md,ms,mb},
		QuantumNumbers -> {Q -> -1/3},
		WeylComponents -> {dqL,dqR}},

(*************** Ghosts ***************)
  U[1] == {
		ClassName -> ghA,
		SelfConjugate -> False,
		Ghost -> A,
		Mass -> 0,
		QuantumNumbers -> {GhostNumber -> 1}},
  U[2] == {
		ClassName -> ghZ,
		SelfConjugate -> False,
		Mass -> MZ,
		Ghost -> Z,
		QuantumNumbers -> {GhostNumber -> 1}},
  U[31] == {
		ClassName -> ghWp,
		SelfConjugate -> False,
		Mass -> MW,
		Ghost -> W,
		QuantumNumbers -> {Q-> 1, GhostNumber -> 1}},
  U[32] == {
		ClassName -> ghWm,
		SelfConjugate -> False,
		Mass -> MW,
		Ghost -> Wbar,
		QuantumNumbers -> {Q-> -1, GhostNumber -> 1}},
  U[4] == {
		ClassName -> ghG,
		SelfConjugate -> False,
		Indices -> {Index[Gluon]},
		Ghost -> G,
		Mass -> 0,
		QuantumNumbers -> {GhostNumber -> 1}},
  U[5] == {
		ClassName -> ghWi,
		Unphysical -> True,
		Definitions -> {ghWi[1] -> (ghWp + ghWm)/Sqrt[2],
						              ghWi[2] -> (ghWm - ghWp)/Sqrt[2]/I,
						              ghWi[3] -> cw etaZ ghZ + etat sw ghA},
		SelfConjugate -> False,
		Ghost -> Wi,
		Indices -> {Index[SU2W]},
		FlavorIndex -> SU2W},
  U[6] == {
		ClassName -> ghB,
		SelfConjugate -> False,
		Definitions -> {ghB -> - etat etaZ sw ghZ + cw ghA},
		Indices -> {},
		Ghost -> B,
		Unphysical -> True},

(*************** Gauge Bosons ***************)
	(* Gauge bosons: Q = 0 *)
  V[1] == {
		ClassName -> A,
		SelfConjugate -> True,
		Mass -> 0},
  V[2] == {
		ClassName -> Z,
		SelfConjugate -> True, 
		Mass -> MZ},
  V[3] == {
		ClassName -> W,
		SelfConjugate -> False,
		Mass -> MW,
		QuantumNumbers -> {Q -> 1}},
  V[4] == {
		ClassName -> G,
		SelfConjugate -> True,
		Indices -> {Index[Gluon]},
		Mass -> 0},
  V[5] == {
		ClassName -> Wi,
		Unphysical -> True,
		Definitions -> {Wi[mu_, 1] -> (W[mu] + Wbar[mu])/Sqrt[2],
		                			  Wi[mu_, 2] -> (Wbar[mu] - W[mu])/Sqrt[2]/I,
		                			  Wi[mu_, 3] -> etaZ cw Z[mu] + etat sw A[mu]},
		SelfConjugate -> True,
		Indices -> {Index[SU2W]},
		FlavorIndex -> SU2W},
  V[6] == {
		ClassName -> B,
		SelfConjugate -> True,
		Definitions -> {B[mu_] -> - etat etaZ sw Z[mu] + cw A[mu]},
		Unphysical -> True},

(*************** Scalars ***************)
  S[1] == {
		ClassName -> H,
		SelfConjugate -> True,
		Mass -> MH},
  S[2] == {
		ClassName -> phi,
		SelfConjugate -> True,
		Mass -> MZ,
		Goldstone -> Z},
  S[3] == {
		ClassName -> phi2,
		SelfConjugate -> False,
		Mass -> MW,
		Goldstone -> W,
		QuantumNumbers -> {Q -> 1}}
}