(* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
This is the Definitions.m file, to be used in the FeynCalc environment.

Here we make some useful definitions and replacement rules.

Created by: Duarte Fontes
Email: duartefontes@tecnico.ulisboa.pt
Last update: 04.10.2019
- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *)

(* ---------------- Some tricks concerning Passarino-Veltman ---------------- *)
PaVeTrick = {PaVe[0, {}, {x_}]->A0[x], PaVe[0, {x_}, {y_,z_}] -> B0[x,y,z], PaVe[1, {x_}, {y_,z_}] -> B1[x,y,z]}
MyPaVeReduce[x_]:= ((((PaVeReduce[x] /. {D -> 4}) + GetFinite[PaVeReduce[x]])//FCE)/.PaVeTrick)//DiracSimplify

(* ---------------- Källén function ---------------- *)
MyKallen[x_,y_,z_]:= Sqrt[x^2+y^2+z^2- 2(x*y + x*z + y*z)]

(* ---------------- MyTeXForm: function to convert expressions into LaTeX style ---------------- *)
dirFM=StringDrop[dirNuc, -8];
MyTeXForm = Function[exp, Module[{},
	str = OpenWrite["dataFC.m"];
	WriteString[str, InputForm[exp // FCE]];
    Close[str];
	If[osswitch=="Windows",
			str = OpenWrite["MyBatch.bat"];
			WriteString[str, "@echo off \n"];
			WriteString[str, "move dataFC.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
			WriteString[str, "cd ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move FRtoTeX.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "cd ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "py JustConvert.py \n"];
		    WriteString[str, "del dataFC.m \n"];
		    WriteString[str, "move FRtoTeX.m ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move MyTeXForm-last-output.tex ",StringReplace[dirHome,"/"->"\\"],"\n"];
		    WriteString[str, "rmdir __pycache__ /S /Q \n"];
		    WriteString[str, "cd ",StringReplace[dirFM,"/"->"\\"],"\n"];
		    WriteString[str, "rmdir __pycache__ /S /Q \n"];
		    (* WriteString[str, "pause \n"]; *)
		    Close[str];
		    Run["MyBatch.bat"];
    		DeleteFile["MyBatch.bat"];
    		ReadString["MyTeXForm-last-output.tex"],
	If[osswitch=="Linux"||osswitch=="Mac",
			str = OpenWrite["MyBatch.sh"];
			WriteString[str, "mv dataFC.m ",dirNuc,"\n"];
			WriteString[str, "cd ",dirFey,"\n"];
		    WriteString[str, "mv FRtoTeX.m ",dirNuc,"\n"];
		    WriteString[str, "cd ",dirNuc,"\n"];
		    WriteString[str, "python3 JustConvert.py \n"];
		    WriteString[str, "rm dataFC.m \n"]; 
		    WriteString[str, "mv FRtoTeX.m ",dirFey,"\n"];
		    WriteString[str, "mv MyTeXForm-last-output.tex ",dirHome,"\n"];
		    WriteString[str, "rm -fr __pycache__  \n"];
		    WriteString[str, "cd ",dirFM,"\n"];
		    WriteString[str, "rm -fr __pycache__  \n"];
		    Close[str];
		    Run["sh MyBatch.sh"];
    		DeleteFile["MyBatch.sh"];
    		ReadString["MyTeXForm-last-output.tex"]]]]];

(* ------------------- FCtoLT: function to convert expressions into Fortran style ------------------- *)
FCtoLT = Function[exp, Module[{},
	str = OpenWrite["dataLT.m"];
	WriteString[str, InputForm[exp // FCE]];
    Close[str];
	str = OpenWrite["auxLT.m"];
	MyFCList = Import[StringJoin[dirFey , "FRtoTeX.m"], "List"];
	MyListaux = {};
	Do[MyListaux = Append[MyListaux, StringDrop[StringSplit[MyFCList[[i]], "->"][[1]], -1]], {i, 1, Length[MyFCList]}];
	FCeqsrev={};
	Do[If[!NumberQ[FCeqsaux[[i,2]]]&&Length[Level[FCeqsaux[[i,2]], 1]]==0&&!MemberQ[MyListaux,ToString[FCeqsaux[[i,2]]]],
			FCeqsrev=Append[FCeqsrev,ToString[FCeqsaux[[i,2]]]->FCeqsaux[[i,1]]]],{i,1,Length[FCeqsaux]}];
	Do[WriteString[str,FCeqsrev[[i]],"\n"],{i,1,Length[FCeqsrev]}];
	Close[str];
	If[osswitch=="Windows",
			str = OpenWrite["MyBatch.bat"];
			WriteString[str, "@echo off \n"];
			WriteString[str, "move dataLT.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
			WriteString[str, "move auxLT.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
			WriteString[str, "cd ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move FRtoTeX.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "move ParamsValues.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "move Matrixind.m ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "cd ",StringReplace[dirNuc,"/"->"\\"],"\n"];
		    WriteString[str, "py JustConvert.py \n"];
		    WriteString[str, "del dataLT.m \n"];
		    WriteString[str, "del auxLT.m \n"];
		    WriteString[str, "move FRtoTeX.m ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move ParamsValues.m ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move Matrixind.m ",StringReplace[dirFey,"/"->"\\"],"\n"];
		    WriteString[str, "move FunctionLT.F ",StringReplace[dirHome,"/"->"\\"],"\n"];
		    WriteString[str, "move MainLT.F ",StringReplace[dirHome,"/"->"\\"],"\n"];
		    WriteString[str, "move MyParameters.h ",StringReplace[dirHome,"/"->"\\"],"\n"];
		    WriteString[str, "rmdir __pycache__ /S /Q \n"];
		    WriteString[str, "cd ",StringReplace[dirFM,"/"->"\\"],"\n"];
		    WriteString[str, "rmdir __pycache__ /S /Q \n"];
		    (* WriteString[str, "pause \n"]; *)
		    Close[str];
		    Run["MyBatch.bat"];
    		DeleteFile["MyBatch.bat"],
	If[osswitch=="Linux"||osswitch=="Mac", 
			str = OpenWrite["MyBatch.sh"];
			WriteString[str, "mv dataLT.m ",dirNuc,"\n"];
			WriteString[str, "mv auxLT.m ",dirNuc,"\n"];
			WriteString[str, "cd ",dirFey,"\n"];
		    WriteString[str, "mv FRtoTeX.m ",dirNuc,"\n"];
		    WriteString[str, "mv ParamsValues.m ",dirNuc,"\n"];
		    WriteString[str, "mv Matrixind.m ",dirNuc,"\n"];
		    WriteString[str, "cd ",dirNuc,"\n"];
		    WriteString[str, "python3 JustConvert.py \n"];
		    WriteString[str, "rm dataLT.m \n"];
		    WriteString[str, "rm auxLT.m \n"];
		    WriteString[str, "mv FRtoTeX.m ",dirFey,"\n"];
		    WriteString[str, "mv ParamsValues.m ",dirFey,"\n"];
		    WriteString[str, "mv Matrixind.m ",dirFey,"\n"];
		    WriteString[str, "mv FunctionLT.F ",dirHome,"\n"];
		    WriteString[str, "mv MainLT.F ",dirHome,"\n"];
		    WriteString[str, "mv MyParameters.h ",dirHome,"\n"];
		    WriteString[str, "rm -fr __pycache__  \n"];
		    WriteString[str, "cd ",dirFM,"\n"];
		    WriteString[str, "rm -fr __pycache__  \n"];
		    Close[str];
		    Run["sh MyBatch.sh"];
    		DeleteFile["MyBatch.sh"]]]]];

(* ------------------- Some useful rules and definitions concerning Dirac matrices and momenta ------------------- *)
distG5 = {x_ .(((1 - GA[5])/2)) -> x/2 - (x.GA[5])/2, x_ .(((1 + GA[5])/2)) -> x/2 + (x.GA[5])/2};
ProjP=(1+dm[5])/2; ProjM=(1-dm[5])/2;
stilldot = {FV[p_, mu_]*GA[mu_] -> GS[p], GA[mu_].GA[5]*FV[p_, mu_] -> GS[p].GA[5]};
myfad = {FAD[x_, Dimension -> 4] -> 1/SP[x, x], FAD[{x_, z_}] -> 1/(SP[x, x]-z)};
expsp = {SP[x_ + y_, z_ + w_] -> SP[x, z] + SP[x, w] + SP[y, z] + SP[y, w],
			SP[-x_,-y_]->SP[x,y], SP[-x_,y_]->-SP[x,y], SP[x_,-y_]->-SP[x,y]};

(* ------------------- Some useful definitions concerning the intervening particles ------------------- *)
Inparticles:=ToString[inparticlesA]
NumberIn:=Length[inparticlesB]
If[ToString[ToExpression[outparticlesA] // InputForm] != "outparticlesA",
	Outparticles:=ToString[outparticlesA];
	NumberOut:=Length[outparticlesB]]

(* ------------------- Momentum conservation ------------------- *)
MomCons={};
If[NumberIn == 1 &&  NumberOut == 1, MomCons={FV[q1, x_] -> FV[p1, x], q1 -> p1}];
If[NumberIn == 1 &&  NumberOut == 2, MomCons={FV[p1, x_] -> FV[q1, x] + FV[q2, x], GS[p1] -> GS[q1] + GS[q2], p1 -> q1 + q2}];
If[NumberIn == 2 &&  NumberOut == 1, MomCons={FV[q1, x_] -> FV[p1, x] + FV[p2, x], GS[q1] -> GS[p1] + GS[p2], q1 -> p1 + p2}];
If[NumberIn == 2 &&  NumberOut == 2, MomCons={FV[q1, x_] -> FV[p1, x] + FV[p2, x] - FV[q2, x], GS[q1] -> GS[p1] + GS[p2] - GS[q2], q1 -> p1 + p2 - q2}];
ApplyMomCons={};
If[MoCoLogic == True,ApplyMomCons=MomCons];

(* ------------------- Function to compute the Decay Width ------------------- *)
If[NumberIn == 1 &&  NumberOut == 2,
DecayWidth = Function[{exp}, 
												  Module[{kin,ApplyTrace,indrepint1,indrepint2,indrep1a,
												  					indrep2a,indrep3a,indrep1b,indrep2b,indrep3b,
												  					XX0,XX0c,XX1,XX2,p1m,inifac,finfac,BarSqModM,DW},								  					
M0 = ToExpression["m"<> ToString[inparticlesB2[[1]]]];
m1 = ToExpression["m"<> ToString[outparticlesB2[[1]]]];
m2 = ToExpression["m"<> ToString[outparticlesB2[[2]]]];
ApplyTrace = {DiracTrace[x_] -> TrG5[x]};
kin={SP[p1, p1] -> M0^2, SP[q1, q1] -> m1^2, SP[q2, q2] -> m2^2,SP[p1, q1] -> 1/2(M0^2+m1^2-m2^2),
          SP[p1, q2] -> 1/2(M0^2+m2^2-m1^2), SP[q1, q2] -> 1/2(M0^2-m1^2-m2^2)};
indrep1a={};indrep2a={};indrep3a={};indrep1b={};indrep2b={};indrep3b={};
indrepint1={$MU[1]->indm1,$MU[2]->indm2,$MU[3]->indm3,$MU[4]->indm4,J1->ind1,J2->ind2,J3->ind3,J4->ind4,J5->ind5,J6->ind6,J7->ind7,J8->ind8,J9->ind9,J10->ind10,J11->ind11,J12->ind12};
indrepint2={indm1->indm1b,indm2->indm2b,indm3->indm3b,indm4->indm4b,ind1->ind1b,ind2->ind2b,ind3->ind3b,ind4->ind4b,ind5->ind5b,ind6->ind6b,ind7->ind7b,ind8->ind8b,ind9->ind9b,ind10->ind10b,ind11->ind11b,ind12->ind12b};
If[ToString[intypes[[1]]]=="gauge"||ToString[intypes[[1]]]=="gluon", indrep1a={-J1->\[Mu]a}; indrep1b={\[Mu]a->\[Mu]b}];
If[ToString[outtypes[[1]]]=="gauge"||ToString[outtypes[[1]]]=="gluon", indrep2a={-J2->\[Nu]a}; indrep2b={\[Nu]a->\[Nu]b}];
If[ToString[outtypes[[2]]]=="gauge"||ToString[outtypes[[2]]]=="gluon", indrep3a={-J4->\[Sigma]a}; indrep3b={\[Sigma]a->\[Sigma]b}];
	If[ToString[intypes[[1]]]=="fermion" || ToString[outtypes[[1]]]=="fermion" || ToString[outtypes[[1]]]=="fermion",
	If[LoSpinors == False, Print["Error: LoSpinors should be set to True to compute the decay width"]; Abort[]]];
	If[MoCoLogic == True, Print["Error: MoCoLogic should be set to False to compute the decay width"]; Abort[]];
XX0 = exp /. indrep1a /. indrep2a /. indrep3a /. indrepint1;
XX0c = ComplexConjugate[XX0]  /. indrep1b /. indrep2b /. indrep3b /. indrepint2;
XX1 = ((ChangeTo4[FermionSpinSum[XX0c XX0]]) /. ApplyTrace) // FCE;
If[ToString[intypes[[1]]]=="gauge"||ToString[intypes[[1]]]=="gluon",
	If[ToString[InputForm[M0]] != "0",
		XX1 = Simplify[XX1*(-MT[\[Mu]a, \[Mu]b] + FV[p1, \[Mu]a] FV[p1, \[Mu]b]/M0^2) // Calc] // FCE,
		XX1 = Simplify[XX1*(-MT[\[Mu]a, \[Mu]b]) // Calc] // FCE],
	XX1 = Simplify[XX1 // Calc] // FCE];
If[ToString[outtypes[[1]]]=="gauge"||ToString[outtypes[[1]]]=="gluon",
	If[ToString[InputForm[m1]] != "0",
		XX1 = Simplify[XX1*(-MT[\[Nu]a, \[Nu]b] + FV[q1, \[Nu]a] FV[q1, \[Nu]b]/m1^2) // Calc] // FCE,
		XX1 = Simplify[XX1*(-MT[\[Nu]a, \[Nu]b]) // Calc] // FCE],
	XX1 = Simplify[XX1 // Calc] // FCE];
If[ToString[outtypes[[2]]]=="gauge"||ToString[outtypes[[2]]]=="gluon",
	If[ToString[InputForm[m2]] != "0",
		XX1 = Simplify[XX1*(-MT[\[Sigma]a, \[Sigma]b] + FV[q2, \[Sigma]a] FV[q2, \[Sigma]b]/m1^2) // Calc] // FCE,
		XX1 = Simplify[XX1*(-MT[\[Sigma]a, \[Sigma]b]) // Calc] // FCE],
	XX1 = Simplify[XX1 // Calc] // FCE];
XX2 = Simplify[XX1 /. kin];
(* Ok, what we did so far was the sum over spins and polarizations of the squared modulus of the total amplitude *)
(* Now, the rest *)
p1m=MyKallen[M0^2,m1^2,m2^2]/(2 M0);
If[ToString[intypes[[1]]]=="fermion",inifac=1/2,inifac=1];
If[ToString[outparticlesB[[1]]]==ToString[outparticlesB[[2]]],finfac=1/2,finfac=1];
BarSqModM=inifac*XX2*finfac;
DW=(4 Pi)*(1/(32 Pi^2))*p1m/(M0^2)*BarSqModM]]];

(* ------------------- Function to compute the differencial cross section ------------------- *)
If[NumberIn == 2 &&  NumberOut == 2,
DiffXS = Function[{exp}, 
												  Module[{kin,ApplyTrace,Ene1,Ene2,Ene3,Ene4,p1m,p2m,p3m,p4m,
												  					indrep1a,indrep2a,indrep3a,indrep4a,
												  					indrep1b,indrep2b,indrep3b,indrep4b,
												  					indrepint1,indrepint2,
												  					XX0,XX0c,XX1pre,XX1,XX2,inifac,finfac,BarSqModM,DXS},
m1 = ToExpression["m"<> ToString[inparticlesB2[[1]]]];
m2 = ToExpression["m"<> ToString[inparticlesB2[[2]]]];
m3 = ToExpression["m"<> ToString[outparticlesB2[[1]]]];
m4 = ToExpression["m"<> ToString[outparticlesB2[[2]]]];
ApplyTrace = {DiracTrace[x_] -> TrG5[x]};
Ene1 = (S+m1^2-m2^2)/(2*Sqrt[S]);
Ene2 = (S+m2^2-m1^2)/(2*Sqrt[S]);
Ene3 = (S+m3^2-m4^2)/(2*Sqrt[S]);
Ene4 = (S+m4^2-m3^2)/(2*Sqrt[S]);
p1m = MyKallen[S,m1^2,m2^2]/(2*Sqrt[S]);
p2m = MyKallen[S,m1^2,m2^2]/(2*Sqrt[S]);
q1m = MyKallen[S,m3^2,m4^2]/(2*Sqrt[S]);
q2m = MyKallen[S,m3^2,m4^2]/(2*Sqrt[S]);
kin={SP[p1, p1] -> m1^2, SP[p2, p2] -> m2^2, SP[q1, q1] -> m3^2, SP[q2, q2] -> m4^2,
          SP[p1, p2] -> Ene1 Ene2 + p1m p2m, 
          SP[p1, q1] -> Ene1 Ene3 - p1m q1m Cos[Theta],
          SP[p1, q2] -> Ene1 Ene4 + p1m q2m Cos[Theta],
          SP[p2, q1] -> Ene2 Ene3 + p2m q1m Cos[Theta],
          SP[p2, q2] -> Ene2 Ene4 - p1m q2m Cos[Theta],
          SP[q1, q2] -> Ene3 Ene4 + q1m q2m};
indrep1a={};indrep2a={};indrep3a={};indrep4a={};indrep1b={};indrep2b={};indrep3b={};indrep4b={};
indrepint1={$MU[1]->indm1,$MU[2]->indm2,$MU[3]->indm3,$MU[4]->indm4,J1->ind1,J2->ind2,J3->ind3,J4->ind4,J5->ind5,J6->ind6,J7->ind7,J8->ind8,J9->ind9,J10->ind10,J11->ind11,J12->ind12};
indrepint2={indm1->indm1b,indm2->indm2b,indm3->indm3b,indm4->indm4b,ind1->ind1b,ind2->ind2b,ind3->ind3b,ind4->ind4b,ind5->ind5b,ind6->ind6b,ind7->ind7b,ind8->ind8b,ind9->ind9b,ind10->ind10b,ind11->ind11b,ind12->ind12b};
If[ToString[intypes[[1]]]=="gauge"||ToString[intypes[[1]]]=="gluon", indrep1a={-J1->\[Mu]a}; indrep1b={\[Mu]a->\[Mu]b}];
If[ToString[intypes[[2]]]=="gauge"||ToString[intypes[[2]]]=="gluon", indrep2a={-J3->\[Rho]a}; indrep2b={\[Rho]a->\[Rho]b}];
If[ToString[outtypes[[1]]]=="gauge"||ToString[outtypes[[1]]]=="gluon", indrep3a={-J2->\[Nu]a}; indrep3b={\[Nu]a->\[Nu]b}];
If[ToString[outtypes[[2]]]=="gauge"||ToString[outtypes[[2]]]=="gluon", indrep4a={-J4->\[Sigma]a}; indrep4b={\[Sigma]a->\[Sigma]b}];
	If[ToString[intypes[[1]]]=="fermion" || ToString[intypes[[2]]]=="fermion" || ToString[outtypes[[1]]]=="fermion" || ToString[outtypes[[1]]]=="fermion",
	If[LoSpinors == False, Print["Error: LoSpinors should be set to True to compute the differential cross section"]; Abort[]]];
	If[MoCoLogic == True, Print["Error: MoCoLogic should be set to False to compute the differential cross section"]; Abort[]];
XX0 = exp /. indrep1a /. indrep2a /. indrep3a /. indrep4a /. indrepint1;
XX0c = ComplexConjugate[XX0]  /. indrep1b /. indrep2b /. indrep3b /. indrep4b /. indrepint2;
XX1pre = ((ChangeTo4[FermionSpinSum[XX0c XX0]]) /. ApplyTrace) // FCE;
XX1pre2 = ((XX1pre /. myfad) /. expsp) /. expsp;
XX1=XX1pre2;
If[ToString[intypes[[1]]]=="gauge"||ToString[intypes[[1]]]=="gluon",
	If[ToString[InputForm[m1]] != "0",
		XX1 = (XX1*(-MT[\[Mu]a, \[Mu]b] + FV[p1, \[Mu]a] FV[p1, \[Mu]b]/m1^2) // Calc) // FCE,
		XX1 = (XX1*(-MT[\[Mu]a, \[Mu]b]) // Calc) // FCE],
	XX1 = (XX1 // Calc) // FCE];
If[ToString[intypes[[2]]]=="gauge"||ToString[intypes[[2]]]=="gluon",
	If[ToString[InputForm[m2]] != "0",
		XX1 = (XX1*(-MT[\[Rho]a, \[Rho]b] + FV[p2, \[Rho]a] FV[p2, \[Rho]b]/m2^2) // Calc) // FCE,
		XX1 = (XX1*(-MT[\[Rho]a, \[Rho]b]) // Calc) // FCE],
	XX1 = (XX1 // Calc) // FCE];	
If[ToString[outtypes[[1]]]=="gauge"||ToString[outtypes[[1]]]=="gluon",
	If[ToString[InputForm[m3]] != "0",
		XX1 = (XX1*(-MT[\[Nu]a, \[Nu]b] + FV[q1, \[Nu]a] FV[q1, \[Nu]b]/m3^2) // Calc) // FCE,
		XX1 = (XX1*(-MT[\[Nu]a, \[Nu]b]) // Calc) // FCE],
	XX1 = (XX1 // Calc) // FCE];
If[ToString[outtypes[[2]]]=="gauge"||ToString[outtypes[[2]]]=="gluon",
	If[ToString[InputForm[m4]] != "0",
		XX1 = (XX1*(-MT[\[Sigma]a, \[Sigma]b] + FV[q2, \[Sigma]a] FV[q2, \[Sigma]b]/m4^2) // Calc) // FCE,
		XX1 = (XX1*(-MT[\[Sigma]a, \[Sigma]b]) // Calc) // FCE],
	XX1 = (XX1 // Calc) // FCE];
XX2 = Simplify[XX1 /. kin];
(* Ok, what we did so far was the sum over spins and polarizations of the squared modulus of the total amplitude. Now, the rest *)
If[ToString[intypes[[1]]]=="fermion"&&ToString[intypes[[2]]]=="fermion",inifac=1/4,
If[ToString[intypes[[1]]]=="fermion"&&ToString[intypes[[2]]]!="fermion",inifac=1/2,
If[ToString[intypes[[1]]]!="fermion"&&ToString[intypes[[2]]]=="fermion",inifac=1/2,inifac=1]]];
If[ToString[outparticlesB[[1]]]==ToString[outparticlesB[[2]]],finfac=1/2,finfac=1];
BarSqModM=inifac*XX2*finfac;
DXS=(1/(64 Pi^2 S))*q1m/p1m*BarSqModM]]];

(* ----------- Two important functions: GetAns and GetRes ----------- *)
(* Note: the (1/I) in the prefactors that follow assure that the final result is M, and not iM *)
Prefac:= (1/I) (I/(16 Pi^2)) (1/(I Pi^2))
PrefacTree := (1/I)
If[loops==0,
	GetAns[amp_,k_]:= Simplify[(PrefacTree Calc[ChangeTo4[amp]]) // SUNSimplify],
	If[LoopTec==1,
		GetAns[amp_,k_]:= Prefac OneLoop[k,amp] // SUNSimplify,
		If[LoopTec==2,
			GetAns[amp_,k_]:= Prefac MyOneLoop[k,amp] // SUNSimplify,
			If[LoopTec==3,
				GetAns[amp_,k_]:= Prefac MyOneLoopMod[k,amp] // SUNSimplify,
				If[LoopTec==4,
					GetAns[amp_,k_]:= Prefac OneLoopTID[k,amp] // SUNSimplify]]]]];
GetRes[ans_]:=((1/factor) * ((ChangeTo4[ans]) + ChangeTo4[GetFinite[ans]]))
