# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - Standard Model - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
content = {
	# - - - - - - Start by writing the 'internal' name of the model, to be used both in the QGRAF model and the Feynrules and Feyncalc folders
	'intname' : 'StandardModel',
	# - - - - - - Write the 'external' name of the model, to be printed in the final pdfs
	'extname' : 'the Standard Model',
	# - - - - - - Write the particles of your model in QGRAF style (the antiparticles should be in the respective order)
	'nscl' : ['H'],    # - - - neutralscalars
	'cscl' : [''],    # - - - chargedscalars
	'ancscl' : [''],    # - - - antichargedscalars
	'fel' : ['d','u','s','c','b','t','e1','e2','e3','n1','n2','n3'],    # - - - fermions
	'anfel'  : ['D','U','S','C','B','T','E1','E2','E3','N1','N2','N3'],    # - - - antifermions
	'ngal' : ['Z','A'],    # - - - neutralgauge
	'cgal' : ['WP'],    # - - - chargedgauge
	'ancgal' : ['WM'],    # - - - antichargedgauge
	'ngol' : ['G0'],    # - - - neutralgoldstones
	'cgol' : ['GP'],    # - - - chargedgoldstones
	'ancgol' : ['GM'],    # - - - antichargedgoldstones
	'ghl' : ['cWP','cWM','cA','cZ','cG'],    # - - - ghosts
	'anghl' : ['CWP','CWM','CA','CZ','CG'],    # - - - antighosts
	'glul' : ['g'],    # - - - gluon
	# - - - - - - Write now their names in LaTeX style, in respective order
	'LAnscl' : ['h'],
	'LAcscl' : [''],
	'LAancscl' : [''],
	'LAfel' : ['d','u','s','c','b','t','e','\\mu','\\tau','\\nu_e','\\nu_{\\mu}','\\nu_{\\tau}'],
	'LAanfel'  : ['\\bar{d}','\\bar{u}','\\bar{s}','\\bar{c}','\\bar{b}','\\bar{t}','\\bar{e}','\\bar{\\mu}','\\bar{\\tau}','\\bar{\\nu_e}','\\bar{\\nu_{\\mu}}','\\bar{\\nu_{\\tau}}'],
	'LAngal' : ['Z','\\gamma'],
	'LAcgal' : ['W^{+}'],
	'LAancgal' : ['W^{-}'],
	'LAngol' : ['G_{0}'],
	'LAcgol' : ['G^{+}'],
	'LAancgol' : ['G^{-}'],
	'LAghl' : ['c_{W^{+}}','c_{W^{-}}','c_{\\gamma}','c_Z','c_g'],
	'LAanghl' : ['\\bar{c_{W^{+}}}','\\bar{c_{W^{-}}}','\\bar{c_{\\gamma}}','\\bar{c_Z}','\\bar{c_g}'],
	'LAglul' : ['g'],
	# - - - - - - Choose the replacement rules for the particles names from FeynRules to QGRAF  (else, leave it '{}'):
	'FRtoQ' : '{e -> e1, m -> e2, tt -> e3, ve -> n1, vm -> n2, vt -> n3, W -> WP, phi -> G0, phi2 -> GP, ghWp -> cWP, ghWm -> cWM, ghA -> cA, ghZ -> cZ, ghG -> cG, G -> g}',
	# - - - - - - Choose a restrictions file for FeynRules (else, leave it ''):
	'FRrestr' : 'MyRestrictions.rst',
	# - - - - - - Choose simplifications for FeynCalc (else, leave it '{}'):
	'FCsimp' : '{cw^2+sw^2->1}',	
	# - - - - - - Choose equalities for FeynCalc (else, leave it []):
	'FCeqs' : ['etae=1','eta1=1','eta2=1','eta3=1','etat=1','etaZ=1','ee=e','vv = 2 MW sw /ee','xiW=1','xiZ=1','xiA=1','\[Lambda] = MH^2*ee^2/(8 MW^2 sw^2)'],	
	# - - - - - - Set PrMassFL to True if you want the propagators masses to be extracted from the Lagrangean; else, they will be a simple version (in the Feynman gauge):
	'PrMassFL' : True,
	# - - - - - - The following variables concern renormalization
	# - - - - - - In rensimp, you can insert simplifications to be made immediately before renormalization (else, leave it '{}')
	# - - - - - - In renconsnum, you should write the renormalization constants which are numbers, not matrices (else, leave it []);
	# - - - - - - then, in LArenconsnum, the same, but in LaTeX style, in respective order (else, leave it []);
	# - - - - - - in renconsmat, you should write the renormalization constants which are squared matrices (else, leave it []);
	# - - - - - - then, in LArenconsmat, the same, but in LaTeX style, in respective order (else, leave it []);
	# - - - - - - in renconsmatdim, you should write the dimension of the squared matrices just defined (else, leave it []);
	# - - - - - - GFreno should be set to True if you want the renormalization rules to be applied to the Gauge Fixing Lagrangean; else, False;
	# - - - - - - finally, in renorrules, the renormalization rules (else, leave it [])
	'rensimp' : '{muH^2 -> 3 vv^2 \[Lambda] - MH^2, \[Lambda] -> MH^2*ee^2/(8 MW^2 sw^2) + tad*ee^3/(16 MW^3 sw^3), vv^2 -> 4 sw^2 MW^2/ee^2, vv^3 -> (8*MW^3*sw^3)/ee^3, (ee*vv)/(MW*sw) -> 2, (MH^2*MW*sw/ee) -> MH^2*vv/2, MW^2/cw^2 -> MZ^2}',	
	'renconsnum' : ['\\[Delta]ZAA', '\\[Delta]ZAZ', '\\[Delta]ZZZ', '\\[Delta]ZZA', '\\[Delta]ZW', '\\[Delta]ZH',
						 			'\\[Delta]Zphi', '\\[Delta]Zphi2', '\\[Delta]ghA', '\\[Delta]ghZ', '\\[Delta]ghWp',
						 			'\\[Delta]md','\\[Delta]ms','\\[Delta]mb','\\[Delta]mu','\\[Delta]mc','\\[Delta]mt','\\[Delta]me1','\\[Delta]me2','\\[Delta]me3',
						 			'\\[Delta]ee', '\\[Delta]MW2', '\\[Delta]MZ2', '\\[Delta]MH2', '\\[Delta]cw', '\\[Delta]sw', '\\[Delta]tad'],
	'LArenconsnum' : ['\\delta Z_{AA}', '\\delta Z_{AZ}', '\\delta Z_{ZZ}', '\\delta Z_{ZA}', '\\delta Z_{W}', '\\delta Z_{H}',
						 			'\\delta Z_{\\phi}', '\\delta Z_{\\phi 2}', '\\delta c_{A}', '\\delta c_{Z}', '\\delta c_{W^+}',
						 			'\\delta m_d', '\\delta m_s', '\\delta m_b', '\\delta m_u', '\\delta m_c', '\\delta m_t', '\\delta m_e', '\\delta m_{\\mu}', '\\delta m_{\\tau}', 
						 			'\\delta e', '\\delta m_W^2', '\\delta m_Z^2', '\\delta m_H^2', '\\delta c_w', '\\delta s_w', '\\delta t'],
	'renconsmat' : ['\\[Delta]ZdqL', '\\[Delta]ZdqR', '\\[Delta]ZuqL', '\\[Delta]ZuqR', 
						 		   '\\[Delta]ZlL', '\\[Delta]ZlR', '\\[Delta]ZvlL', '\\[Delta]ZvlR',
						 		   '\\[Delta]yl','\\[Delta]yd','\\[Delta]yu','\\[Delta]CKM'],
	'LArenconsmat' : ['\\delta Z^{dq,L}', '\\delta Z^{dq,R}', '\\delta Z^{uq,L}', '\\delta Z^{uq,R}', 
						 		   '\\delta Z^{l,L}', '\\delta Z^{l,R}', '\\delta Z^{\\nu,L}', '\\delta Z^{\\nu,R}',
						 		   '\\delta Y^{l}','\\delta Y^{d}','\\delta Y^{u}','\\delta V'],
	'renconsmatdim' : ['3','3','3','3','3','3','3','3','3','3','3','3'],
	'GFreno' : True,
	'renorrules' : ['A[\\[Nu]_] -> A[\\[Nu]] + (1/2)  \\[Delta]ZAA A[\\[Nu]] +  (1/2) \\[Delta]ZAZ Z[\\[Nu]]',
							   'Z[\\[Nu]_] -> Z[\\[Nu]] + (1/2)  \\[Delta]ZZZ Z[\\[Nu]] + (1/2) \\[Delta]ZZA A[\\[Nu]]',
							   'W[\\[Nu]_] -> W[\\[Nu]] + (1/2)  \\[Delta]ZW W[\\[Nu]]',
							   'Wbar[\\[Nu]_] -> Wbar[\\[Nu]] + (1/2)  \\[Delta]ZW Wbar[\\[Nu]]',
							   'H -> H + (1/2)  \\[Delta]ZH H',
							   'phi -> phi', 'phibar -> phibar', 'phi2 -> phi2', 'phi2bar -> phi2bar',
							   'ghA -> ghA', 'ghAbar -> ghAbar', 'ghZ -> ghZ', 'ghZbar -> ghZbar',
							   'ghWp -> ghWp', 'ghWpbar -> ghWpbar', 'ghWm -> ghWm', 'ghWmbar -> ghWmbar',
							   #
							   'dqL -> dqL + (1/2)  \\[Delta]ZdqL.dqL',
							   'dqR -> dqR + (1/2)  \\[Delta]ZdqR.dqR',
							   'uqL -> uqL + (1/2)   \\[Delta]ZuqL.uqL',
							   'uqR -> uqR + (1/2)   \\[Delta]ZuqR.uqR',
							   'lL -> lL + (1/2)   \\[Delta]ZlL.lL',
							   'lR -> lR + (1/2)   \\[Delta]ZlR.lR',
							   'vlL -> vlL + (1/2)   \\[Delta]ZvlL.vlL',
							   'vlR -> vlR + (1/2)   \\[Delta]ZvlR.vlR',
							   #
							   'dqLbar -> dqLbar + (1/2)   dqLbar.HC[\\[Delta]ZdqL]',
							   'dqRbar -> dqRbar + (1/2)   dqRbar.HC[\\[Delta]ZdqR]',
							   'uqLbar -> uqLbar + (1/2)   uqLbar.HC[\\[Delta]ZuqL]',
							   'uqRbar -> uqRbar + (1/2)   uqRbar.HC[\\[Delta]ZuqR]',
							   'lLbar -> lLbar + (1/2)   lLbar.HC[\\[Delta]ZlL]',
							   'lRbar -> lRbar + (1/2)   lRbar.HC[\\[Delta]ZlR]',
							   'vlLbar -> vlLbar + (1/2)   vlLbar.HC[\\[Delta]ZvlL]',
							   'vlRbar -> vlRbar + (1/2)   vlRbar.HC[\\[Delta]ZvlR]',
							   #
							   'yl -> yl +  \\[Delta]yl',
							   'yd -> yd +  \\[Delta]yd',
							   'yu -> yu +  \\[Delta]yu',
							   #
							   'ee -> ee +  \\[Delta]ee ee',
							   'cw -> cw + \\[Delta]cw',
							   'sw -> sw + \\[Delta]sw',
							   'tad -> tad + \\[Delta]tad',
							   'MW -> MW +  \\[Delta]MW2/(2 MW)',
							   'MZ -> MZ +  \\[Delta]MZ2/(2 MZ)',
							   'MH -> MH +  \\[Delta]MH2/(2 MH)',
							   'CKM -> CKM +  \\[Delta]CKM']
}