(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
This is the Preamble.m file, to be used in the FeynRules environment.

Here we make some useful definitions and lists.

Created by: Duarte Fontes
Email: duartefontes@tecnico.ulisboa.pt
Last update: 03.08.2019
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

(* Some initial instructions *)
 FR$DSign = -1;
 G5 := ProjP - ProjM

(* Creation of adapted particles lists *)
aux1 = {};
palist = {};
Do[Do[aux1 = Append[aux1, StringTake[ToString[PartList[[i, 1, 1]]], 1]];
	If[Length[PartList[[i, 2]]] > 1,
	If[StringTake[ToString[PartList[[i, 2, j, 1]]], -1]=="-"||StringTake[ToString[PartList[[i, 2, j, 1]]], -1]=="+",
	aux1 = Append[aux1, StringDrop[ToString[PartList[[i, 2, j, 1]]], -1]],
	aux1 = Append[aux1, ToString[PartList[[i, 2, j, 1]]]]],
	aux1 = Append[aux1, ToString[PartList[[i, 1, 2]]]]];
	palist = Append[palist, aux1];
	aux1 = {}, {j, 1, Length[PartList[[i, 2]]]}], {i, 1,Length[PartList]}];
palist2 = {};
Do[If[palist[[i, 1]] != "W", palist2 = Append[palist2, palist[[i]]]];, {i, 1, Length[palist]}];
palistq = palist2;
palist2 = palist2 // ToExpression;
pinc={};
Do[pinc=Append[pinc,FRtoQ[[i,1]]],{i,1,Length[FRtoQ]}]; 
Do[If[!MemberQ[pinc,palist2[[i,2]]],FRtoQ=Append[FRtoQ,palist2[[i,2]]->palist2[[i,2]]]],{i,1,Length[palist2]}];
FRtoQin = FRtoQ;
Do[Do[If[ToString[FRtoQin[[i, 2]]] == ToString[antimap[[j, 2]]], 
    FRtoQ = Append[FRtoQ, ToExpression[ToString[FRtoQin[[i, 1]]] <> "bar -> " <> ToString[antimap[[j, 1]]]]]],
    		{j, 1, Length[antimap]}], {i, 1,Length[FRtoQin]}];
palist3 = palist2 /.FRtoQ;
palist4 = {};
Do[palist4 = Append[palist4, {}], {i, 1, Length[palist3]}];
Do[palist4[[i]] = Append[palist4[[i]], ToString[palist3[[i, 1]]]];
  palist4[[i]] = Append[palist4[[i]], ToString[palist3[[i, 2]]]], {i, 1, Length[palist3]}];

(* A list concerning propagators *)
auxprop = {};
auxprop2 = {};
Do[auxprop = Append[auxprop, {}], {i, 1, Length[palist4]}];
Do[auxprop2 = Append[auxprop2, {}], {i, 1, Length[palist4]}];
Do[Do[Do[If[FRtoQ[[m, n]] == ToExpression[palistq[[i, 2]] <> "bar"], 
     auxprop2[[i]] = Append[auxprop2[[i]], m]], {n, 1, Length[FRtoQ[[m]]]}], {m, 1, Length[FRtoQ]}], {i, 1, Length[palist4]}];
Do[
  auxprop[[i]] = Append[auxprop[[i]], palist4[[i, 2]]];
  auxprop[[i]] = Append[auxprop[[i]], If[auxprop2[[i]] != {}, FRtoQ[[auxprop2[[i, 1]], 2]], palist4[[i, 2]]]];
  auxprop[[i]] = Append[auxprop[[i]], If[palist4[[i, 1]] == "F" || palist4[[i, 1]] == "U", "-", "+"]], {i, 1, Length[palist4]}];

(* A function to separate parcels of an expression into a list *)
Parc=Function[exp,Module[{auxp},
auxp=Replace[exp, HoldPattern[Plus[a___]] :> {a}];
auxp]];
(* A function to contract Lorentz indices *)
MyContract = Function[{exp}, Module[{x0, auxc, x1},
    x0 = ExpandIndices[exp];
    auxc = {ME[a_, b_]*ME[b_, c_] -> ME[a, c], FV[p_, K_] FV[p_, K_] -> p^2, FV[p_, Index[Lorentz, x_]] -> FV[p, x]};
    x1 = x0 /. auxc;
    x1]];
(* A function to solve equations *)
MySolve = Function[{exp, var}, Module[{x0, x1, x2}, x0 = Coefficient[exp, var];
    x1 = x0 var;
    x2 = var -> -(exp - x1)/x0;
    x2]];
(* A way to avoid the anoying Sqrt[x^2]: *)
mysqrt=Function[exp,Module[{auxp1,auxp2},
auxp1=Sqrt[exp];
auxp2=auxp1/.{Sqrt[x_^2]->x, Sqrt[x_^2*y_]->x*Sqrt[y]};
auxp2]];

(* Convert the parameters to their LaTeX names *)
paramstotex = {};
matrixind={};
Do[Do[If[ToString[InputForm[M$Parameters[[i, 2, j, 1]]]] == "TeX", 
    				paramstotex = Append[paramstotex, ToString[InputForm[M$Parameters[[i, 1]]]]
    				-> ToString[InputForm[TeXForm[M$Parameters[[i, 2, j, 2]]]]]]],{j,1, Length[M$Parameters[[i, 2]]]}], {i, 1, Length[M$Parameters]}];
Do[
MyCol=1;
MyRow=1;
Do[If[ToString[InputForm[M$Parameters[[i, 2, k, 1]]]] == "Indices",
						If[Length[M$Parameters[[i, 2, k, 2]]]==2,
						MyCol=Length[IndexRange[M$Parameters[[i, 2, k, 2, 1]]]];
						MyRow=Length[IndexRange[M$Parameters[[i, 2, k, 2, 2]]]],
						MyCol=Length[IndexRange[M$Parameters[[i, 2, k, 2, 1]]]]];
						matrixind = Append[matrixind,{M$Parameters[[i,1]],MyCol,MyRow}]],
						{k,1, Length[M$Parameters[[i, 2]]]}], {i, 1, Length[M$Parameters]}];

Do[paramstotex[[i,2]]=StringReplace[paramstotex[[i,2]]," _"->"_"];
	   paramstotex[[i,2]]=StringReplace[paramstotex[[i,2]],"\\text"->""];
paramstotex[[i,2]]=StringReplace[paramstotex[[i,2]],paramstotex[[i,2]]->"{"<>paramstotex[[i,2]]<>"}"],{i,1,Length[paramstotex]}];
If[LArenconsadd!={},Do[paramstotex = Append[paramstotex, ToString[InputForm[renconsadd[[i]]]] -> StringReplace[LArenconsadd[[i]],"\\\\"->"\\"]],{i,1,Length[LArenconsadd]}]];

(* List with the indices that show up in the parameters *)
IndicesList = {};
Do[Do[
			  If[StringContainsQ[(M$Parameters[[i, 2]] // InputForm) // ToString, "Indices"], 
    			  If[StringContainsQ[(M$Parameters[[i, 2, j]] // InputForm) // ToString, "Indices"], 
     				  If[! MemberQ[IndicesList, (M$Parameters[[i, 2, j, 2]] // InputForm) // ToString], 
      					  IndicesList = Append[IndicesList, (M$Parameters[[i, 2, j, 2]] // InputForm) // ToString]]]],
      		 {j, 1, Length[M$Parameters[[i, 2]]]}], {i, 1, Length[M$Parameters]}];
fileIL = OpenWrite["IndicesList.m"];
Do[WriteString[fileIL, IndicesList[[i]]];
        If[i != Length[IndicesList], WriteString[fileIL, "\n"]], {i, 1, Length[IndicesList]}];
Close[fileIL];

(* Lists to deal with masses in different conventions *)
massmap1 = {};
Do[If[StringLength[ToString[FRtoQ[[i, 1]]]] < 4, 
   		massmap1 = Append[massmap1, ToExpression[ToString[FRtoQ[[i, 2]]] <> "->" <> ToString[FRtoQ[[i, 1]]]]], 
   If[StringTake[ToString[FRtoQ[[i, 1]] // InputForm], -3] != "bar", 
    	massmap1 = Append[massmap1, ToExpression[ToString[FRtoQ[[i, 2]]] <> "->" <> ToString[FRtoQ[[i, 1]]]]]]], {i, 1, Length[FRtoQ]}];
massmap2 = {};
Do[membercoin = 0;
  Do[If[ToString[InputForm[M$ClassesDescription[[i, 2, p, 1]]]] == "ClassMembers", membercoin = 1], {p, 1, Length[M$ClassesDescription[[i, 2]]]}];
  Do[If[ToString[InputForm[M$ClassesDescription[[i, 2, j, 1]]]] == "Mass",
  				If[membercoin == 0,
  						Do[If[ToString[InputForm[M$ClassesDescription[[i, 2, k, 1]]]] == "ClassName", 
       							massmap2 = Append[massmap2, ToExpression[ToString[M$ClassesDescription[[i, 2, k, 2]]] <> "->" <> ToString[M$ClassesDescription[[i, 2, j, 2]]]]]],
       							{k, 1, Length[M$ClassesDescription[[i, 2]]]}], 
     					Do[If[ToString[InputForm[M$ClassesDescription[[i, 2, k, 1]]]] == "ClassMembers", 
       							Do[massmap2 = Append[massmap2, ToExpression[ToString[M$ClassesDescription[[i, 2, k, 2, l]]] <> "->" <> ToString[M$ClassesDescription[[i, 2, j, 2, l]]]]],
       							{l, 1, Length[M$ClassesDescription[[i, 2, k, 2]]]}]], {k, 1, Length[M$ClassesDescription[[i, 2]]]}]]], {j, 1, Length[M$ClassesDescription[[i, 2]]]}], {i, 1, Length[M$ClassesDescription]}];
masstrade = {};
Do[Do[If[ToString[massmap1[[i, 2]]] == ToString[massmap2[[j, 1]]],
		masstrade = Append[masstrade, ToExpression["m" <> ToString[massmap1[[i, 1]]]	<> "->" <> ToString[massmap2[[j, 2]]]]]],
					{j, 1, Length[massmap2]}], {i,1, Length[massmap1]}];
masstotex = {};
Do[Do[If[ToString[masstrade[[i, 2]]] == ToString[paramstotex[[j, 1]]],
     masstotex = Append[masstotex, ToString[masstrade[[i, 1]]] -> paramstotex[[j, 2]]]], {j, 1, Length[paramstotex]}], {i, 1, Length[masstrade]}];

(* Lists with the numerical values of the parameters *)
paramstovals = {};
Do[Do[If[ToString[InputForm[M$Parameters[[i, 2, j, 1]]]] == "Value", 
    paramstovals = Append[paramstovals, ToString[InputForm[M$Parameters[[i, 1]]] ] -> ToString[M$Parameters[[i, 2, j, 2]]]]],
    		{j, 1, Length[M$Parameters[[i, 2]]]}], {i, 1, Length[M$Parameters]}];
Do[Do[If[ToString[InputForm[M$Parameters[[i, 2, j, 1]]]] == "Values", 
    paramstovals = Append[paramstovals, ToString[InputForm[M$Parameters[[i, 1]]] ] -> ToString[M$Parameters[[i, 2, j, 2]]]]],
    		{j, 1, Length[M$Parameters[[i, 2]]]}], {i, 1, Length[M$Parameters]}];
paramstovals2 = paramstovals;

(* Output file with the numerical values of the parameters *)
str = OpenWrite["ParamsValues.m"];
Do[
If[Length[ToExpression[paramstovals2[[i,2]]]]==0,
WriteString[str,paramstovals2[[i,1]]<>"="<>paramstovals2[[i,2]]<>"\n"],
WriteString[str,paramstovals2[[i,1]]<>"="<>StringDrop[StringDrop[ToString[paramstovals2[[i,2]]],1],-1]<>"\n"]],{i,1,Length[paramstovals2]}];
Close[str];

(* List with some of the restrictions of the restrictions file *)
MyRestr = {};
Do[If[! StringContainsQ[ToString[M$Restrictions[[i, 1]] // InputForm],"_"], 
			MyRestr = Append[MyRestr, ToExpression[StringReplace[ToString[InputForm[M$Restrictions[[i]]]],":>" -> "->"]]]], {i, 1, Length[M$Restrictions]}];

(* List with the rotation of the gauge fields to their mass eigenstates *)
SU2exp = {};
Do[Do[If[ToString[InputForm[M$ClassesDescription[[i, 2, j, 1]]]] == "Definitions" && ToString[InputForm[M$ClassesDescription[[i, 1, 0]]]] == "V", 
    Do[SU2exp = Append[SU2exp, M$ClassesDescription[[i, 2, j, 2, k]]], {k, 1, Length[M$ClassesDescription[[i, 2, j, 2]]]}]],
    	{j, 1, Length[M$ClassesDescription[[i, 2]]]}], {i, 1, Length[M$ClassesDescription]}];

(* Tricks concerning the projectors: *)
notensdots = {TensDot[Ga[Index[Lorentz, mu_]], ProjM] -> Ga[mu].ProjM, TensDot[Ga[Index[Lorentz, mu_]], ProjP] -> Ga[mu].ProjP};
todots = {ProjM Ga[Index[Lorentz, mu_]] -> Ga[Index[Lorentz, mu]].ProjM, ProjP Ga[Index[Lorentz, mu_]] -> Ga[Index[Lorentz, mu]].ProjP};
assodots = {x_ .ProjM + x_ .ProjP -> x};
pprops = {ProjM + ProjP -> 1};

(* Some tricks concerning colour *)
mygluind={};
colcoin=0;
myglu="";
myglugh="";
Do[Do[Do[Do[
     Do[If[ToString[InputForm[M$GaugeGroups[[h0, 2, h1a, 1]]]] ==  "StructureConstant"
     				&& ToString[InputForm[M$GaugeGroups[[h0, 2, h1a, 2]]]] == "f"
     				&& ToString[InputForm[M$ClassesDescription[[j0, 2, j1, 1]]]] == "ClassName"
     				&& ToString[InputForm[M$GaugeGroups[[h0, 2, h1b, 1]]]] == "GaugeBoson"
     				&& ToString[InputForm[M$GaugeGroups[[h0, 2, h1b, 2]]]] == ToString[InputForm[M$ClassesDescription[[j0, 2, j1, 2]]]], 
       					colcoin += 1;
       					myglu = ToString[ToExpression[ToString[InputForm[M$GaugeGroups[[h0, 2, h1b, 2]]]]] /. FRtoQ];
       					Do[If[ToString[InputForm[M$ClassesDescription[[j0, 2, m, 1]]]]=="Indices",
       								mygluind=M$ClassesDescription[[j0, 2, m, 2]]],{m,1,Length[M$ClassesDescription[[j0, 2]]]}];
						Do[If[StringTake[ToString[InputForm[M$ClassesDescription[[n0, 1]]]], 1] == "U",
								Do[If[ToString[InputForm[M$ClassesDescription[[n0, 2, n1, 1]]]]=="Indices"&&mygluind!={}&&M$ClassesDescription[[n0, 2, n1, 2]]==mygluind,
									Do[If[ToString[InputForm[M$ClassesDescription[[n0, 2, n2, 1]]]]=="ClassName",
									myglugh = ToString[ToExpression[ToString[InputForm[M$ClassesDescription[[n0, 2, n2, 2]]]]] /. FRtoQ]],
									{n2,1,Length[M$ClassesDescription[[n0,2]]]}]],{n1,1,Length[M$ClassesDescription[[n0,2]]]}]],{n0,1,Length[M$ClassesDescription]}]],
       					{j1, 1, Length[M$ClassesDescription[[j0, 2]]]}],
       					{j0,1, Length[M$ClassesDescription]}],
       					{h1b, 1, Length[M$GaugeGroups[[h0, 2]]]}],
       					{h1a, 1, Length[M$GaugeGroups[[h0, 2]]]}],
       					{h0, 1, Length[M$GaugeGroups]}];

(* Rules to create the Feynman rules from bilinears: *)
proprep = {};
Do[Do[
If[palist[[i, 1]] == "S" && palist[[j, 1]] == "S",
If[palist[[i, 2]]==palist[[j, 2]],
If[ToString[ToString[StandardForm[ToExpression[palist[[i, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[i, 2]]<>"bar"]]]],
proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]], \[Mu]_]) (del[ToExpression[palist[[j, 2]]], \[Mu]_]) -> 2 I p^2];
proprep = Append[proprep, (ToExpression[palist[[i, 2]]]) (ToExpression[palist[[j, 2]]]) -> 2 I],
	proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]<>"bar"], \[Mu]_]) (del[ToExpression[palist[[j, 2]]], \[Mu]_]) -> I p^2];
	proprep = Append[proprep, (ToExpression[palist[[i, 2]]<>"bar"]) (ToExpression[palist[[j, 2]]]) -> I]],
	If[ToString[ToString[StandardForm[ToExpression[palist[[i, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[i, 2]]<>"bar"]]]] && \
        ToString[ToString[StandardForm[ToExpression[palist[[j, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[j, 2]]<>"bar"]]]],
			proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]], \[Mu]_]) (del[ToExpression[palist[[j, 2]]], \[Mu]_]) -> I p^2];
			proprep = Append[proprep, (ToExpression[palist[[i, 2]]]) (ToExpression[palist[[j, 2]]]) -> I],
				proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]<>"bar"], \[Mu]_]) (del[ToExpression[palist[[j, 2]]], \[Mu]_]) -> I p^2];
				proprep = Append[proprep, (ToExpression[palist[[i, 2]]<>"bar"]) (ToExpression[palist[[j, 2]]]) -> I]]],
					If[palist[[i, 1]] == "V" && palist[[j, 1]] == "V",
					If[palist[[i, 2]]==palist[[j, 2]],
					If[ToString[ToString[StandardForm[ToExpression[palist[[i, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[i, 2]]<>"bar"]]]],
						If[ToString[(palist[[i, 2]] // ToExpression) /. FRtoQ] == myglu,
							proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Mu]_, a_], \[Mu]_]^2 -> 2 I fv[p, -J1] fv[p, -J2]];
							proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Mu]_, a_], \[Mu]_] del[ToExpression[palist[[i, 2]]][\[Nu]_, a_], \[Nu]_] -> 2 I fv[p, -J1] fv[p, -J2]];
							proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_, a_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Nu]_, a_], \[Mu]_] -> 2 I mt[-J1, -J2] p^2];
							proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_, a_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Mu]_, a_], \[Nu]_] -> 2 I fv[p, -J1] fv[p, -J2]],
								proprep = Append[proprep, ToExpression[palist[[i, 2]]][\[Mu]_]*ToExpression[palist[[j, 2]]][\[Mu]_] -> 2 I mt[-J1, -J2]];
								proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Mu]_], \[Mu]_]^2 -> 2 I fv[p, -J1] fv[p, -J2]];
								proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Mu]_], \[Mu]_] del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Nu]_] -> 2 I fv[p, -J1] fv[p, -J2]];
								proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Nu]_], \[Mu]_] -> 2 I mt[-J1, -J2] p^2];
								proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Mu]_], \[Nu]_] -> 2 I fv[p, -J1] fv[p, -J2]]],
									proprep = Append[proprep, ToExpression[palist[[i, 2]]<>"bar"][\[Mu]_]*ToExpression[palist[[j, 2]]][\[Mu]_] -> I mt[-J1, -J2]];
									proprep = Append[proprep, del[ToExpression[palist[[i, 2]]<>"bar"][\[Mu]_], \[Mu]_]*del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Nu]_] -> I fv[p, -J1] fv[p, -J2]];
									proprep = Append[proprep, del[ToExpression[palist[[i, 2]]<>"bar"][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Nu]_], \[Mu]_] -> I mt[-J1, -J2] p^2];
									proprep = Append[proprep, del[ToExpression[palist[[i, 2]]<>"bar"][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Mu]_], \[Nu]_] -> I fv[p, -J1] fv[p, -J2]]],
										If[ToString[ToString[StandardForm[ToExpression[palist[[i, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[i, 2]]<>"bar"]]]] && \
										    ToString[ToString[StandardForm[ToExpression[palist[[j, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[j, 2]]<>"bar"]]]],
												proprep = Append[proprep, ToExpression[palist[[i, 2]]][\[Mu]_]*ToExpression[palist[[j, 2]]][\[Mu]_] -> I mt[-J1, -J2]];
												proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Mu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Nu]_], \[Nu]_] -> I fv[p, -J1] fv[p, -J2]];
												proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Nu]_], \[Mu]_] -> I mt[-J1, -J2] p^2];
												proprep = Append[proprep, del[ToExpression[palist[[i, 2]]][\[Nu]_], \[Mu]_]*del[ToExpression[palist[[j, 2]]][\[Mu]_], \[Nu]_] -> I fv[p, -J1] fv[p, -J2]]]],
													If[palist[[i, 1]] == "F" && palist[[j, 1]] == "F", 
													proprep = Append[proprep, ToExpression[palist[[i, 2]] <> "bar . " <> palist[[j, 2]]] -> I];
													proprep = Append[proprep, ToExpression[palist[[i, 2]] <> "bar . del[" <> palist[[j, 2]] <> ",Index[Lorentz,\[Mu]_]]"] -> ToExpression["fv[p,\[Mu]]"]],
														If[palist[[i, 1]] == "U" && palist[[j, 1]] == "U",
															If[ToString[(palist[[i, 2]] // ToExpression) /. FRtoQ] == myglugh,
																proprep = Append[proprep, ToExpression[palist[[i, 2]]<>"bar"][a_].(del[del[ToExpression[palist[[j, 2]]][a_], \[Mu]_], \[Mu]_]) -> - I p^2];
																proprep = Append[proprep, (ToExpression[palist[[i, 2]]<>"bar"][a_]) (ToExpression[palist[[j, 2]]][a_]) -> I],
																	proprep = Append[proprep, ToExpression[palist[[i, 2]]<>"bar"].(del[del[ToExpression[palist[[j, 2]]], \[Mu]_], \[Mu]_]) -> - I p^2];
																	proprep = Append[proprep, (ToExpression[palist[[i, 2]]<>"bar"]) (ToExpression[palist[[j, 2]]]) -> I]],
																		If[palist[[i, 1]] == "S" && palist[[j, 1]] == "V",
																			If[ToString[ToString[StandardForm[ToExpression[palist[[i, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[i, 2]]<>"bar"]]]] && \
																			ToString[ToString[StandardForm[ToExpression[palist[[j, 2]]<>"bar"]]]] == ToString[ToString[InputForm[ToExpression[palist[[j, 2]]<>"bar"]]]],
																			proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]], \[Mu]_]) ToExpression[palist[[j, 2]]][\[Mu]_] -> I fv[p, -J1]];
																			proprep = Append[proprep, ToExpression[palist[[i, 2]]] del[ToExpression[palist[[j, 2]]][\[Mu]_], \[Mu]_] -> - I fv[p, -J1]],
																				proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]<>"bar"], \[Mu]_]) ToExpression[palist[[j, 2]]][\[Mu]_] -> I fv[p, -J1]];
																				proprep = Append[proprep, ToExpression[palist[[i, 2]]<>"bar"] del[ToExpression[palist[[j, 2]]][\[Mu]_], \[Mu]_] -> I fv[p, -J1]];
																				proprep = Append[proprep, (del[ToExpression[palist[[i, 2]]], \[Mu]_]) ToExpression[palist[[j, 2]]<>"bar"][\[Mu]_] -> I fv[p, -J1]];
																				proprep = Append[proprep, ToExpression[palist[[i, 2]]] del[ToExpression[palist[[j, 2]]<>"bar"][\[Mu]_], \[Mu]_] -> I fv[p, -J1]]]]]]]], {j,1,Length[palist]}],{i, 1, Length[palist]}];

(* Convert the propagators FeynCalc-style notation to FeynRules notation *)
myprops = {fv -> FV, mt -> ME}

(* Ok; what follows concerns the \[Eta]s: a very useful tool to obtain Feynman rules for propagators and counterterms *)
(* Create a different \[Eta] for each particle: *)
fixetas={};
Do[fixetas=Append[fixetas,ToExpression["\[Eta]" <> ToString[palist2[[i,2]]]]];
If[ToString[ToString[StandardForm[ToExpression[ToString[palist2[[i, 2]]]<>"bar"]]]] != ToString[ToString[InputForm[ToExpression[ToString[palist2[[i, 2]]]<>"bar"]]]],
fixetas=Append[fixetas,ToExpression["\[Eta]" <> ToString[palist2[[i,2]]] <>"bar"]]],{i,1,Length[palist2]}];
(* Insert the \[Eta]s in the equations: *)
insetas={};
Do[
insetas = Append[insetas, palist2[[i,2]] -> palist2[[i,2]] ToExpression["\[Eta]" <> ToString[palist2[[i,2]]]]];
If[ToString[ToString[StandardForm[ToExpression[ToString[palist2[[i, 2]]]<>"bar"]]]] != ToString[ToString[InputForm[ToExpression[ToString[palist2[[i, 2]]]<>"bar"]]]],
insetas = Append[insetas, ToExpression[ToString[palist2[[i, 2]]]<>"bar"] ->
										ToExpression[ToString[palist2[[i, 2]]]<>"bar"] ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]]<>"bar"]]],{i,1,Length[palist2]}];
(* Set the derivatives of the \[Eta]s to zero *)
consrules = {};
Do[consrules = Append[consrules, del[fixetas[[i]], \[Mu]_] -> 0], {i, 1, Length[fixetas]}];
(* Take the \[Eta]s out of the fermionic products: *)
repsetas={};
Do[
repsetas = Append[repsetas, (fixetas[[i]] x_) . y_ -> fixetas[[i]] (x.y)];
repsetas = Append[repsetas, x_ . (fixetas[[i]] y_) -> fixetas[[i]] (x.y)],{i,1,Length[fixetas]}];
repsetas = Append[repsetas, (\[Eta]\[Delta] x_) . y_ -> \[Eta]\[Delta] (x.y)];
repsetas = Append[repsetas, x_ . (\[Eta]\[Delta] y_) -> \[Eta]\[Delta] (x.y)];
(* Take the \[Eta]s out of the 4vector definiti0ns: *)
stilletas={};
Do[
stilletas = Append[stilletas, (x_ fixetas[[i]])[mu_] -> fixetas[[i]] x[mu]];
stilletas = Append[stilletas, (x_ fixetas[[i]])[mu_,a_] -> fixetas[[i]] x[mu,a]],{i,1,Length[fixetas]}];
(* Take all the \[Eta]s to zero: *)
cleanetas = {};
Do[cleanetas = Append[cleanetas, fixetas[[i]] ->0], {i, 1, Length[fixetas]}];
(* Take all the \[Eta]s to one: *)
unitetas = {};
Do[unitetas = Append[unitetas, fixetas[[i]] ->1], {i, 1, Length[fixetas]}];
(* Convert, in the case of the antiparticles in the QGRAF convention, the capital letters to the 'bar' convention *)
antietas = {};
Do[If[ToString[auxprop[[i,1]]] != ToString[auxprop[[i,2]]],
antietas = Append[antietas, ToExpression["\[Eta]" <> ToString[auxprop[[i, 2]]]] -> ToExpression["\[Eta]" <> ToString[auxprop[[i, 1]]] <> "bar"]]], {i, 1, Length[auxprop]}];
(* Convert the \[Eta]s in the QGRAF convention (or 'bar' convention) to the Feynrules convention:*)
convetas={};
Do[convetas = Append[convetas, ToExpression["\[Eta]" <> ToString[palist3[[i, 2]]]] -> ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]]]];
If[MemberQ[fixetas,ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]] <> "bar"]],
convetas = Append[convetas, ToExpression["\[Eta]" <> ToString[palist3[[i, 2]]] <> "bar"] -> ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]] <> "bar"]]],{i,1,Length[palist2]}];
(* prebacks are going to be useful to convert combetas (in renormalization) into a list (CTord) crucial to print the counterterms *)
preback1={};
Do[preback1 = Append[preback1, convetas[[i,2]] -> convetas[[i,1]]],{i,1,Length[convetas]}];
preback2={};
Do[preback2 = Append[preback2, antietas[[i,2]] -> antietas[[i,1]]],{i,1,Length[antietas]}];
preback3={};
Do[preback3 = Append[preback3, preback1[[i,1]] -> (preback1[[i,2]] /. preback2)], {i,1,Length[preback1]}];
comeback={};
Do[comeback = Append[comeback, preback3[[i,1]] -> ToExpression[StringDrop[preback3[[i, 2]] // ToString, 1]]],{i,1,Length[preback3]}];
(* Distribute the fermionic product: *)
dotexpa = {(a_ + b_).x_ -> a.x + b.x, x_ .(a_ + b_) -> x.a + x.b, (a_ + b_ + c_).x_ -> a.x + b.x + c.x, x_ .(a_ + b_ + c_) -> x.a + x.b + x.c};
(* Create \[Eta]s for each propagator: *)
propsetas = {};
Do[
If[ToString[InputForm[palist2[[i, 1]]]]!="F"&&ToString[InputForm[palist2[[i, 1]]]]!="W",
If[ToString[ToString[StandardForm[ToExpression[ToString[palist2[[i, 2]]] <> "bar"]]]] != ToString[ToString[InputForm[ToExpression[ToString[palist2[[i, 2]]] <> "bar"]]]], 
   propsetas = Append[propsetas, ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]] <> "bar"]*ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]]]],
   propsetas = Append[propsetas, ToExpression["\[Eta]" <> ToString[palist2[[i, 2]]]]^2]]], {i, 1, Length[palist2]}];
(* Associate the corresponding type and name of particle to the \[Eta]s for the propagators: *)
propstypes = {};
propsnames = {};
Do[If[ToString[InputForm[palist2[[i, 1]]]]!="F"&&ToString[InputForm[palist2[[i, 1]]]]!="W",
propstypes = Append[propstypes, palist2[[i, 1]]];
propsnames = Append[propsnames, palist4[[i, 2]]]], {i, 1, Length[palist2]}];